//
//  NearbyMessagesApiMCMs.m
//  nearby_messages_api
//
//  Created by Thomas Cross on 3/2/19.
//

#import "NearbyMessagesApiMCMs.h"
#import <Flutter/Flutter.h>
#import "GNSMessages.h"

static NSString * const kBackgroundModeSaveKey = @"NearbyBackgroundEnabled";
static NSString * const kOnFound = @"onFound";
static NSString * const kOnLost = @"onLost";

@interface NearbyMessagesApiMCMs ()
@property(nonatomic) GNSPermission *nearbyPermission;
@property(nonatomic) FlutterMethodChannel *channel;
@end

@implementation NearbyMessagesApiMCMs

// Private
GNSMessageManager *messageMgr;
GNSStrategy *strategy;
id<GNSPublication> publication;
id<GNSSubscription> subscription;


- (instancetype)init {
    return [self initWithChannel:nil];
}

- (instancetype)initWithChannel:(FlutterMethodChannel *)channel {
    self = [super init];
    if (self) {
        self.channel = channel;
    }
    return self;
}

- (void)setAPIKey:(NSString *)apikey {
    messageMgr = [[GNSMessageManager alloc]
                   initWithAPIKey:apikey
                   paramsBlock:^(GNSMessageManagerParams *params) {
        params.microphonePermissionErrorHandler = ^(BOOL hasError) {
            // Update the UI for microphone permission
        };
        params.bluetoothPowerErrorHandler = ^(BOOL hasError) {
            // Update the UI for Bluetooth power
        };
        params.bluetoothPermissionErrorHandler = ^(BOOL hasError) {
            // Update the UI for Bluetooth permission
        };
    }];
    
    [self setStrategy];
}

// Private
- (void)setStrategy {
    strategy = [GNSStrategy strategyWithParamsBlock:^(GNSStrategyParams *params) {
        params.allowInBackground = NO;
        params.discoveryMediums = kGNSDiscoveryMediumsDefault;
        params.discoveryMode = kGNSDiscoveryModeDefault;
    }];
}

- (void)setGranted {
    [GNSPermission setGranted:YES];
}

/// Starts sharing with a randomized name and scanning for others.
- (void)backgroundSubscribe {
    [self unPublish];
    [self backgroundUnsubscribe];
    
    // Subscribe to messages from nearby devices and display them in the message view.
    NSString *(^stringFromData)(NSData *) = ^(NSData *content) {
        return [[NSString alloc] initWithData:content encoding:NSUTF8StringEncoding];
    };
    subscription = [messageMgr
                     subscriptionWithMessageFoundHandler:^(GNSMessage *message) {
        NSString *messageString = stringFromData(message.content);
        
        [self.channel invokeMethod:kOnFound arguments:messageString];
    }
                     messageLostHandler:^(GNSMessage *message) {
        NSString *messageString = stringFromData(message.content);
        
        [self.channel invokeMethod:kOnLost arguments:messageString];
    } paramsBlock:^(GNSSubscriptionParams *params) {
        params.strategy = strategy;
    }];
    
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey:kBackgroundModeSaveKey];
}

- (void)publish:(NSString *) message {
    [self unPublish];
    [self backgroundUnsubscribe];
    
    // Publish the name to nearby devices.
    GNSMessage *pubMessage = [GNSMessage messageWithContent:[message dataUsingEncoding:NSUTF8StringEncoding]];
    
    publication = [messageMgr publicationWithMessage:pubMessage paramsBlock:^(GNSPublicationParams *params) {
        params.strategy = strategy;
    }];
}

- (void)unPublish {
    publication = nil;
}

- (void)backgroundUnsubscribe {
    subscription = nil;
    [[NSUserDefaults standardUserDefaults] setBool:NO forKey:kBackgroundModeSaveKey];
}

- (void)enableDebugMode {
    [GNSMessageManager setDebugLoggingEnabled:YES];
}

- (void)disableDebugMode {
    [GNSMessageManager setDebugLoggingEnabled:NO];
}

@end
